// Read input from stdin for now.
// Might change in the future.
//
// Better not to have self-loops.
//

#include <cstdio>
#include <cstring>
#include <vector>
using namespace std;

const int mod = 1000000007;

// Given.
int n, m;
vector<int> g[1 << 17];
int x, y, l;

// parameters of srg. Not given.
//
int k, a, b;

int nei1[1 << 17];
int neiv[1 << 17];

int common_neighbours(int v) {
  memset(neiv, 0, sizeof(neiv));
  for (int i = 0; i < (int) g[v].size(); ++i) {
    neiv[ g[v][i] ] = 1;
  }
  int r = 0;
  for (int i = 2; i <= n; ++i) {
    r += (nei1[i] & neiv[i]);
  }
  // self-loop
  //
  if (nei1[v] && neiv[v]) {
    --v;
  }
  return r;
}

void matmult(long long c[3][3], long long a[3][3], long long b[3][3]) {
  for (int i = 0; i < 3; ++i) {
    for (int j = 0; j < 3; ++j) {
      c[i][j] = 0;
      for (int k = 0; k < 3; ++k) {
        c[i][j] += a[i][k] * b[k][j];
        if (c[i][j] >= mod) {
          c[i][j] %= mod;
        }
      }
    }
  }
}

void qpow(long long m[3][3], long long st) {
  long long res[3][3];
  memset(res, 0, sizeof(res));
  res[0][0] = res[1][1] = res[2][2] = 1;

  long long t[3][3];

  while (st) {
    if (st & 1) {
      memcpy(t, res, sizeof(t));
      matmult(res, t, m);
    }
    st /= 2;
    matmult(t, m, m);
    memcpy(m, t, sizeof(t));
  }
  memcpy(m, res, sizeof(res));
}

bool is_adj(int x, int y) {
  for (int i = 0; i < (int) g[x].size(); ++i) {
    if (g[x][i] == y) {
      return true;
    }
  }
  return false;
}

int main() {
  scanf("%d %d", &n, &m);
  for (int i = 0; i < m; ++i) {
    int j, k;
    scanf("%d %d", &j, &k);
    g[j].push_back(k);
    g[k].push_back(j);
  }

  scanf("%d %d %d", &x, &y, &l);

  k = g[1].size();

  memset(nei1, 0, sizeof(nei1));
  for (int i = 0; i < (int) g[1].size(); ++i) {
    nei1[ g[1][i] ] = 1;
  }
  a = 0;
  b = 0;
  // Possible to introduce some log factors when computing k, a, b.
  //
  int nei;
  int notnei;
  nei = -1;
  notnei = -1;
  for (int i = 2; i <= n; ++i) {
    if (nei == -1 && nei1[i]) {
      nei = i;
    } else if (notnei == -1 && !nei1[i]) {
      notnei = i;
    }
  }

  if (nei != -1) {
    a = common_neighbours(nei);
  }
  if (notnei != -1) {
    b = common_neighbours(notnei);
  }

  fprintf(stderr, "params: %d %d %d %d\n", n, k, a, b);

  // c[1...l] : no paths if neighbours.
  // d[1...l] : no paths if not neighbours.
  // e[1...l] : no paths that loop.
  // c[1] = 1;
  // d[1] = 0;
  // e[1] = 0;
  // c[n] = a * c[n - 1] + (k - a - 1) * d[n - 1] + e[n - 1]
  // d[n] = b * c[n - 1] + (k - b) * d[n - 1]
  // e[n] = k * c[n - 1]
  
  long long mat[3][3];
  mat[0][0] = a; mat[0][1] = k - a - 1; mat[0][2] = 1;
  mat[1][0] = b; mat[1][1] = k - b;     mat[1][2] = 0;
  mat[2][0] = k; mat[2][1] = 0;         mat[2][2] = 0;
  qpow(mat, l - 1);
  long long res;

  if (x == y) {
    res = mat[2][0];
  } else if (is_adj(x, y)) {
    res = mat[0][0];
  } else {
    res = mat[1][0];
  }
  printf("%d\n", (int) res);

  return 0;
}
